@@ -890,7 +890,7 @@ def list_installed_wago_auras(manager: _manager.Manager) -> None:
     config = BuilderConfig()
     aura_groups = WaCompanionBuilder(manager, config).extract_installed_auras()
     installed_auras = sorted(
-        (g.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
+        (g.Meta.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
         for v in g.__root__.values()
         for a in v
