@@ -14,7 +14,8 @@ from collections.abc import (
 )
 from contextlib import asynccontextmanager, contextmanager
 import contextvars as cv
-from itertools import chain, compress, filterfalse, repeat, starmap
+import datetime
+from itertools import chain, compress, filterfalse, repeat, starmap, takewhile
 import json
 from pathlib import Path, PurePath
 from shutil import copy
@@ -50,7 +51,7 @@ from .utils import (
     bucketise,
     chain_dict,
     file_uri_to_path,
-    find_zip_base_dirs,
+    find_addon_zip_base_dirs,
     gather,
     is_not_stale,
     make_zip_member_filter,
@@ -111,19 +112,13 @@ async def _open_temp_writer() -> AsyncIterator[tuple[Path, Callable[[bytes], Awa
 def _open_pkg_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], None]]]:
     from zipfile import ZipFile
 
-    ZIP_EXCLUDES = {
-        # Mac 'resource forks' are ommitted cuz they violate our one package
-        # per folder policy-contract-thing (besides just being clutter)
-        '__MACOSX',
-    }
-
     with ZipFile(path) as archive:
 
         def extract(parent: Path) -> None:
             archive.extractall(parent, members=filter(make_zip_member_filter(base_dirs), names))
 
         names = archive.namelist()
-        base_dirs = find_zip_base_dirs(names) - ZIP_EXCLUDES
+        base_dirs = set(find_addon_zip_base_dirs(names))
         yield (base_dirs, extract)
 
 
@@ -590,40 +585,67 @@ class Manager:
         self,
         search_terms: str,
         limit: int,
-        sources: Set[str] | None = None,
+        sources: Set[str] = frozenset(),
+        start_date: datetime.datetime | None = None,
     ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
         import rapidfuzz
 
         catalogue = await self.synchronise()
 
-        w = 0.5  # Weighing edit distance and download score equally
+        ew = 0.5
+        dw = 1 - ew
+
         normalise = normalise_names('')
 
-        if sources is None:
+        if not sources:
             sources = self.resolvers.keys()
+        else:
+            unknown_sources = sources - self.resolvers.keys()
+            if unknown_sources:
+                raise ValueError(f'Unknown sources: {", ".join(unknown_sources)}')
+
+        def make_filter():
+            def filter_game_flavour(entry: CatalogueEntry):
+                return self.config.game_flavour in entry.game_flavours
+
+            yield filter_game_flavour
+
+            if sources is not None:
+
+                def filter_sources(entry: CatalogueEntry):
+                    return entry.source in sources
+
+                yield filter_sources
+
+            if start_date is not None:
+                start_date_ = start_date
+
+                def filter_age(entry: CatalogueEntry):
+                    return entry.last_updated >= start_date_
+
+                yield filter_age
+
+        filter_fns = list(make_filter())
 
         s = normalise(search_terms)
+
         tokens_to_entries = bucketise(
-            (
-                (normalise(i.name), i)
-                for i in catalogue.__root__
-                if self.config.game_flavour in i.game_flavours and i.source in sources
-            ),
+            ((normalise(e.name), e) for e in catalogue.__root__ if all(f(e) for f in filter_fns)),
             key=lambda v: v[0],
         )
         matches = rapidfuzz.process.extract(
-            s, list(tokens_to_entries), scorer=rapidfuzz.fuzz.WRatio, limit=limit
+            s, list(tokens_to_entries), scorer=rapidfuzz.fuzz.WRatio, limit=limit * 2
         )
         weighted_entries = sorted(
             (
-                (-((s / 100) * w + i.derived_download_score * (1 - w)), i)
+                (-((s / 100) * ew + e.derived_download_score * dw), e)
                 for m, s, _ in matches
-                for _, i in tokens_to_entries[m]
+                for _, e in tokens_to_entries[m]
             ),
             key=lambda v: v[0],
         )
-        return [e for _, e in weighted_entries]
+        return [e for _, e in weighted_entries[:limit]]
 
     @_with_lock('change state')
     async def install(
@@ -777,3 +799,40 @@ class Manager:
                 return R.PkgStrategyUnsupported(Strategy.version)
 
         return {d: await capture_manager_exc_async(t(pin)(d, self.get_pkg(d))) for d in defns}
+
+
+async def is_outdated() -> tuple[bool, str]:
+    """Check on PyPI to see if the installed copy of instawow is outdated.
+
+    The response is cached for 24 hours.
+    """
+    from aiohttp.client import ClientError
+
+    from . import __version__
+
+    def parse_version(version: str) -> tuple[int, ...]:
+        version_parts = takewhile(lambda p: all(c in '0123456789' for c in p), version.split('.'))
+        return tuple(map(int, version_parts))
+
+    if __version__ == '0.0.0':
+        return (False, '')
+
+    dummy_config = Config.get_dummy_config()
+    if not dummy_config.auto_update_check:
+        return (False, '')
+
+    cache_file = dummy_config.temp_dir / '.pypi_version'
+    if await t(is_not_stale)(cache_file, {'days': 1}):
+        version = cache_file.read_text(encoding='utf-8')
+    else:
+        try:
+            async with init_web_client(raise_for_status=True) as web_client, web_client.get(
+                'https://pypi.org/pypi/instawow/json'
+            ) as response:
+                version = (await response.json())['info']['version']
+        except ClientError:
+            version = __version__
+        else:
+            cache_file.write_text(version, encoding='utf-8')
+
+    return (parse_version(version) > parse_version(__version__), version)
